@@ -1,10 +1,30 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# @Date    : 2020-03-23 19:28:33
-# @Author  : Zhi Liu (zhiliu.mind@gmail.com)
-# @Link    : http://iridescent.ink
-# @Version : $1.0$
-from __future__ import division, print_function, absolute_import
+#-*- coding: utf-8 -*-
+# @file      : sampling.py
+# @author    : Zhi Liu
+# @email     : zhiliu.mind@gmail.com
+# @homepage  : http://iridescent.ink
+# @date      : Sun Nov 11 2019
+# @version   : 0.0
+# @license   : The Apache License 2.0
+# @note      : 
+# 
+# The Apache 2.0 License
+# Copyright (C) 2013- Zhi Liu
+#
+#Licensed under the Apache License, Version 2.0 (the "License");
+#you may not use this file except in compliance with the License.
+#You may obtain a copy of the License at
+#
+#http://www.apache.org/licenses/LICENSE-2.0
+#
+#Unless required by applicable law or agreed to in writing, software
+#distributed under the License is distributed on an "AS IS" BASIS,
+#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#See the License for the specific language governing permissions and
+#limitations under the License.
+#
+
 import numpy as np
 from pyaibox.base.arrayops import sl, cat
 from pyaibox.base.randomfunc import setseed, randgrid, randperm
@@ -12,29 +32,29 @@ from pyaibox.base.arrayops import arraycomb
 from pyaibox.utils.ios import loadmat, loadh5
 
 
-def upsampling(X, shape, axis=-1, method='Lanczos'):
+# def upsampling(X, shape, axis=-1, method='Lanczos'):
 
-    # Na, Nr = X.shape
-    # Y = np.zeros(shape, dtype=X.dtype)
-    # for a in range(Na):
-    #     Y[a, :] = np.interp(range(shape[1]), range(Nr), X[a, :])
-    # for r in range(Nr):
-    #     Y[:, a] = np.interp(range(shape[0]), range(Na), X[:, r])
+#     # Na, Nr = X.shape
+#     # Y = np.zeros(shape, dtype=X.dtype)
+#     # for a in range(Na):
+#     #     Y[a, :] = np.interp(range(shape[1]), range(Nr), X[a, :])
+#     # for r in range(Nr):
+#     #     Y[:, a] = np.interp(range(shape[0]), range(Na), X[:, r])
 
-    # return Y
+#     # return Y
 
-    print(shape, X.shape)
-    imgXr = Image.fromarray(X[:, :, 0])
-    # imgXr = imgXr.resize((shape[1], shape[0]), Image.LANCZOS)
-    # imgXr = imgXr.resize((shape[1], shape[0]), Image.BILINEAR)
-    imgXr = imgXr.resize((shape[1], shape[0]), Image.NEAREST)
-    # imgXr = imgXr.resize((shape[1], shape[0]), Image.ANTIALIAS)
-    imgXi = Image.fromarray(X[:, :, 1])
-    # imgXi = imgXi.resize((shape[1], shape[0]), Image.LANCZOS)
-    # imgXi = imgXi.resize((shape[1], shape[0]), Image.BILINEAR)
-    imgXi = imgXi.resize((shape[1], shape[0]), Image.NEAREST)
-    # imgXi = imgXi.resize((shape[1], shape[0]), Image.ANTIALIAS)
-    return np.transpose(np.array([np.array(imgXr), np.array(imgXi)]), (1, 2, 0))
+#     print(shape, X.shape)
+#     imgXr = Image.fromarray(X[:, :, 0])
+#     # imgXr = imgXr.resize((shape[1], shape[0]), Image.LANCZOS)
+#     # imgXr = imgXr.resize((shape[1], shape[0]), Image.BILINEAR)
+#     imgXr = imgXr.resize((shape[1], shape[0]), Image.NEAREST)
+#     # imgXr = imgXr.resize((shape[1], shape[0]), Image.ANTIALIAS)
+#     imgXi = Image.fromarray(X[:, :, 1])
+#     # imgXi = imgXi.resize((shape[1], shape[0]), Image.LANCZOS)
+#     # imgXi = imgXi.resize((shape[1], shape[0]), Image.BILINEAR)
+#     imgXi = imgXi.resize((shape[1], shape[0]), Image.NEAREST)
+#     # imgXi = imgXi.resize((shape[1], shape[0]), Image.ANTIALIAS)
+#     return np.transpose(np.array([np.array(imgXr), np.array(imgXi)]), (1, 2, 0))
 
 
 def slidegrid(start, stop, step, shake=0, n=None):
@@ -91,7 +111,7 @@ def slidegrid(start, stop, step, shake=0, n=None):
 
 
 def dnsampling(x, ratio=1., axis=-1, smode='uniform', omode='discard', seed=None, extra=False):
-    """down-sampling a tensor
+    r"""down-sampling a tensor
 
     Args:
         x (ndarray): The input tensor.
@@ -197,7 +217,7 @@ def dnsampling(x, ratio=1., axis=-1, smode='uniform', omode='discard', seed=None
 
 
 def sample_tensor(x, n, axis=0, groups=1, mode='sequentially', seed=None, extra=False):
-    """sample a tensor
+    r"""sample a tensor
 
     Sample a tensor sequentially/uniformly/randomly.
 
@@ -279,7 +299,7 @@ def sample_tensor(x, n, axis=0, groups=1, mode='sequentially', seed=None, extra=
 
 
 def shuffle_tensor(x, axis=0, groups=1, mode='inter', seed=None, extra=False):
-    """shuffle a tensor
+    r"""shuffle a tensor
 
     Shuffle a tensor randomly.
 
@@ -361,7 +381,7 @@ def shuffle_tensor(x, axis=0, groups=1, mode='inter', seed=None, extra=False):
 
 
 def split_tensor(x, ratios=[0.7, 0.2, 0.1], axis=0, shuffle=False, seed=None, extra=False):
-    """split a tensor
+    r"""split a tensor
 
     split a tensor into some parts.
 
@@ -399,7 +419,7 @@ def split_tensor(x, ratios=[0.7, 0.2, 0.1], axis=0, shuffle=False, seed=None, ex
 
 
 def tensor2patch(x, n=None, size=(256, 256), axis=(0, 1), start=(0, 0), stop=(None, None), step=(1, 1), shake=(0, 0), mode='slidegrid', seed=None):
-    """sample patch from a tensor
+    r"""sample patch from a tensor
 
     Sample some patches from a tensor, tensor and patch can be any size.
 
@@ -427,8 +447,8 @@ def tensor2patch(x, n=None, size=(256, 256), axis=(0, 1), start=(0, 0), stop=(No
         and then reform these patches into an image.
 
         .. image:: ./_static/demo_sample_patch.png
-        :scale: 100 %
-        :align: center
+           :scale: 100 %
+           :align: center
 
         The results shown in the above figure can be obtained by the following codes.
 
@@ -529,7 +549,7 @@ def tensor2patch(x, n=None, size=(256, 256), axis=(0, 1), start=(0, 0), stop=(No
 
 
 def patch2tensor(p, size=(256, 256), axis=(1, 2), mode='nfirst'):
-    """merge patch to a tensor
+    r"""merge patch to a tensor
 
 
     Args:
@@ -589,7 +609,7 @@ def patch2tensor(p, size=(256, 256), axis=(1, 2), mode='nfirst'):
 
 
 def read_samples(datafiles, keys=[['SI', 'ca', 'cr']], nsamples=[10], groups=[1], mode='sequentially', axis=0, parts=None, seed=None):
-    """Read samples
+    r"""Read samples
 
     Args:
         datafiles (list): list of path strings
